package ServerTeamSpeakPackage
{
	function armor::onTrigger(%this, %object, %trigger, %status)
	{
		%r = Parent::onTrigger(%this, %object, %trigger, %status);
		%client = %object.client;
		if(isObject(%client))
		{
			if(%client.isTalking&&%trigger==3)
			{
				if(%status)
				{
					%object.unMountImage(2);
					%object.mountImage(TeamSpeakTalkCrouchImage,2);
				}
				else
				{
					%object.unMountImage(2);
					%object.mountImage(TeamSpeakTalkImage,2);
				}
			}
		}
		return %r;
	}
	function armor::onAdd(%this, %object)
	{

		%r = Parent::onAdd(%this, %object);
		%client = %object.client;
		if(isObject(%client))
		{
			if(%client.isTalking)
			{
				%object.mountImage(TeamSpeakTalkImage,2);
			}
		}
		return %r;
	}
};
activatepackage(ServerTeamSpeakPackage);

datablock ParticleData(TeamSpeakTalkParticle)
{
   dragCoefficient      = 0;
   windCoefficient     = 0;
   gravityCoefficient   = 0;
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.0;
   spinRandomMin = 0;
   spinRandomMax = 0;
   lifetimeMS           = 5;
   lifetimeVarianceMS   = 0;
   textureName          = "Add-Ons/Server_TeamSpeak/teamspeaktalk";
   colors[0]     = "1 1 1 1";
   colors[1]     = "1 1 1 1";
   colors[2]     = "1 1 1 1";
   sizes[0]      = 1;
   sizes[1]      = 1;
   sizes[2]      = 1;
};
datablock ParticleEmitterData(TeamSpeakTalkEmitter)
{
   ejectionPeriodMS = 4;
   periodVarianceMS = 0;
   ejectionVelocity = 0;
   velocityVariance = 0.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 0;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "TeamSpeakTalkParticle";
};
datablock ShapeBaseImageData(TeamSpeakTalkImage)
{
   shapeFile = "base/data/shapes/empty.dts";
	emap = false;

	mountPoint = 2;
   rotation = "1 0 0 -90";
   offset = "0 -0.25 0.8";

	stateName[0]					= "Ready";
	stateTransitionOnTimeout[0]		= "FireA";
	stateTimeoutValue[0]			= 0.01;

	stateName[1]					= "FireA";
	stateTransitionOnTimeout[1]		= "Done";
	stateWaitForTimeout[1]			= True;
	stateTimeoutValue[1]			= 20000;
	stateEmitter[1]					= TeamSpeakTalkEmitter;
	stateEmitterTime[1]				= 20000;

	stateName[2]					= "Done";
	stateScript[2]					= "onDone";
};
datablock ShapeBaseImageData(TeamSpeakTalkCrouchImage)
{
   shapeFile = "base/data/shapes/empty.dts";
	emap = false;

	mountPoint = 2;
   rotation = "1 0 0 -90";
   offset = "0 -1.25 0.8";

	stateName[0]					= "Ready";
	stateTransitionOnTimeout[0]		= "FireA";
	stateTimeoutValue[0]			= 0.01;

	stateName[1]					= "FireA";
	stateTransitionOnTimeout[1]		= "Done";
	stateWaitForTimeout[1]			= True;
	stateTimeoutValue[1]			= 20000;
	stateEmitter[1]					= TeamSpeakTalkEmitter;
	stateEmitterTime[1]				= 20000;

	stateName[2]					= "Done";
	stateScript[2]					= "onDone";
};
function TeamSpeakTalkImage::onDone(%this,%obj,%slot)
{
	%obj.unMountImage(%slot);
}

function initTeamspeakServerCon()
{
	new TCPObject(teamspeakServerCon);
	teamspeakServerCon.connect("localhost:25639");
}
function teamspeakServerCon::onDisconnect(%this)
{
	echo("Disconnected from Teamspeak3 Client. Reconnecting...");
	teamspeakServerCon.schedule(1000, connect, "localhost:25639");
}
function teamspeakServerCon::onConnected(%this)
{
	echo("Connection to Teamspeak3 Client succeeded.");
	%this.schedule(100,com,"serverconnectinfo");
}
function teamspeakServerCon::onConnectFailed(%this)
{
	warn("Connection to Teamspeak3 Client failed!");
	warn("Do you have ClientQuery enabled? Or are even running Teamspeak 3?");
}

function teamspeakServerCon::onLine(%this, %line)
{
	$lastline = %line;
	switch$(getWord(%line,0))
	{
	case "\rerror":
		if(!(getWord(%line,1) $="id=0"))
		{
				warn("Teamspeak Client error \"" @ %line @ "\"");
		}
		return;
	case "\rnotifytalkstatuschange":
		setTeamspeakServerTalk(strReplace(getWord(%line,4),"clid=",""),strReplace(getWord(%line,2),"status=","")); //clientID, isTalking
	case "\rnotifychannelcreated":
		%this.schedule(100,com,"channellist");
	case "\rnotifychanneldeleted":
		%this.schedule(100,com,"channellist");
	case "\rnotifychanneledited":
		%this.schedule(100,com,"channellist");
	case "\rnotifychannelmoved":
		%this.schedule(100,com,"channellist");
	case "\rnotifyclientleftview":
		%this.schedule(100,com,"clientlist");
	case "\rnotifycliententerview":
		%this.schedule(100,com,"clientlist");
	}
	
	if(%this.curCom $= "channellist")
		TSServerbuildChannelList(%line);
	if(%this.curCom $= "clientlist")
		TSServerbuildClientList(%line);
	if(%this.curCom $= "serverconnectinfo")
	{
		%this.callbackEvents();
		%this.schedule(100,com,"channellist");
		%this.schedule(200,com,"clientlist");
	}
	%this.curCom = "";
}
function teamspeakServerCon::com(%this,%com)
{
	%this.curCom = %com;
	%this.send(%com @ "\n");
}
function teamspeakServerCon::callbackEvents(%this)
{
	%this.com("clientnotifyregister schandlerid=0 event=notifytalkstatuschange");
	%this.com("clientnotifyregister schandlerid=0 event=notifychannelcreated");
	%this.com("clientnotifyregister schandlerid=0 event=notifychanneldeleted");
	%this.com("clientnotifyregister schandlerid=0 event=notifychanneledited");
	%this.com("clientnotifyregister schandlerid=0 event=notifychannelmoved");
	%this.com("clientnotifyregister schandlerid=0 event=notifyclientleftview");
	%this.com("clientnotifyregister schandlerid=0 event=notifycliententerview");
	%this.com("clientnotifyregister schandlerid=0 event=notifyconnectioninfo");
}

function setTeamspeakServerTalk(%clid, %isTalking)
{
	%count = ClientGroup.getCount();
	%name = TSServerParseStr(findTSServerName(%clid));
	for(%i=0;%i<%count;%i++)
	{
		%blclient = ClientGroup.getObject(%i);
		if(%blclient.LANname $= %name)
		{
			%blclient.isTalking = %isTalking;
			%blplayer = %blclient.player;
			if(isObject(%blplayer))
			{
				if(%isTalking == 1)
				{
					if(%blplayer.isCrouched())
					{
						%blplayer.mountImage(TeamSpeakTalkCrouchImage,2);
					}
					else
					{
						%blplayer.mountImage(TeamSpeakTalkImage,2);
					}
				}
				else
				{
					%blplayer.unMountImage(2);
				}
			}
		}
	}
}

function TSServerparseStr(%str)
{
	%str = strReplace(%str,"\\s"," ");
	%str = strReplace(%str,"\\p","|");
	%str = strReplace(%str,"\\n"," ");
	%str = strReplace(%str,"\\\/","/");
	if(strPos(%str,"[URL]") != -1)
	{

		if(strPos(%str,"ftp://") != -1)
		{
			%start = strPos(%str,"ftp://");
			%end = strPos(%str,"[/URL]"); 
			%link = getSubStr(%str, %start, %end - %start);
			%str = strReplace(%str,"ftp://","");
		}
		if(strPos(%str,"http://") != -1)
		{
			%start = strPos(%str,"http://");
			%end = strPos(%str,"[/URL]"); 
			%link = getSubStr(%str, %start, %end - %start);
			%str = strReplace(%str,"http://","");
		}
		if(strPos(%str,"https://") != -1)
		{
			%start = strPos(%str,"https://");
			%end = strPos(%str,"[/URL]"); 
			%link = getSubStr(%str, %start, %end - %start);
			%str = strReplace(%str,"https://","");
		}
		%str = strReplace(%str,"[URL]","<a:");
		%str = strReplace(%str,"[/URL]",">" @ %link @ "</a>");
	}
	%str = strReplace(%str,"[b]","");
	%str = strReplace(%str,"[/b]","");
	%str = strReplace(%str,"[i]","");
	%str = strReplace(%str,"[/i]","");
	%str = strReplace(%str,"[u]","");
	%str = strReplace(%str,"[/u]","");
	return %str;
}

function TSServerfindCLID(%clid)
{
	for(%i = 0;%i < teamspeakServerClients.count;%i++)
	{
		if(teamspeakServerClients.client[%i,"id"] == %clid)
			return %i;
	}
	return -1;
}

function findTSServerName(%clid)
{
	for(%i = 0;%i < teamspeakServerClients.count;%i++)
	{
		if(teamspeakServerClients.client[%i,"id"] == %clid)
			return teamspeakServerClients.client[%i,"name"];
	}
	return -1;
}

function TSServerbuildChannelList(%channels)
{
	if(isObject(teamspeakserverChannels))
		teamspeakserverChannels.delete();
	new scriptObject(teamspeakserverChannels);
	%i = 0;
	while(strlen(%channels) > 1)
	{
		%tempChannels = strchr(%channels,"|");
		%chan = getSubStr(%channels,0,strLen(%channels)-strLen(%tempChannels));
		if(%tempChannels !$= "")
			%channels = getSubStr(%tempChannels,1,strLen(%tempChannels)-1);
		else
			%channels = "";
		teamspeakserverChannels.channel[%i,"id"] = strReplace(strReplace(getWord(%chan,0),"cid=",""),"\r","");
		teamspeakserverChannels.channel[%i,"name"] = strReplace(getWord(%chan,3),"channel_name=","");
		%i++;
	}
	teamspeakserverChannels.count = %i;
}

function TSServerbuildClientList(%clients)
{
	//echo("building clist list");
	new scriptObject(newteamspeakServerClients);
	%i = 0;
	while(strlen(%clients) > 1)
	{
		%tempClients = strchr(%clients,"|");
		%cl = getSubStr(%clients,0,strLen(%clients)-strLen(%tempClients));
		if(%tempClients !$= "")
			%clients = getSubStr(%tempClients,1,strLen(%tempClients)-1);
		else
			%clients = "";
		newteamspeakServerClients.client[%i,"id"] = strReplace(strReplace(getWord(%cl,0),"clid=",""),"\r","");
		newteamspeakServerClients.client[%i,"name"] = strReplace(getWord(%cl,3),"client_nickname=","");
		%i++;
	}
	newteamspeakServerClients.count = %i;
	if(isObject(teamspeakServerClients))
	{
		teamspeakServerClients.delete();
	}
	newteamspeakServerClients.setName(teamspeakServerClients);
}

function teamspeakServerConnect()
{
	teamspeakServerCon.connect("localhost:25639");
}

schedule(10,0,initTeamspeakServerCon);